SystemOrganization addCategory: #RSSNewsReader!

Object subclass: #RSSNewsReader
	instanceVariableNames: 'list index contents dropList listIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RSSNewsReader'!

!RSSNewsReader commentStamp: '<historical>' prior: 0!
A RSSNewsReader is a small example application, which demonstrates how GUI applications can be made in Smalltalk with only few lines of code.

Instance Variables
	contents:		GUI contents
	dropList:		list of different news sources (two-dimensional array containing URL and name)
	index:		selected news headline in headline list component
	list:		news headline list component, displays all news items
	listIndex:		selected news source in news source drop list component!

!RSSNewsReader class methodsFor: 'as yet unclassified' stamp: 'MJK 7/11/2013 17:42'!
menuCommandOn: aBuilder
	"Registers automatically the RSSNewsReader to the World Menu."
	<worldMenu>
	(aBuilder item: #'News Reader')
		order: 10;
		action: [self open];
		withSeparatorAfter.! !

!RSSNewsReader class methodsFor: 'as yet unclassified' stamp: 'MJK 7/5/2013 17:17'!
open
	"Creates a new RSSNewsReader object and opens the GUI."
	self new open.! !

!RSSNewsReader methodsFor: 'user-interface' stamp: 'MJK 7/5/2013 17:16'!
buildGUI
	"Constructs a GUI. Fills the 'contents' instance variable with a list of Polymorph objects."
	| builder |
	
	builder := UITheme builder.
	contents := builder newColumn: {
		builder newDropListFor: self
			list: #dropList
			getSelected: #getListIndex
			setSelected: #setListIndex:
			help: nil.
		builder newListFor: self
			list: #list
			selected: #getIndex
			changeSelected: #setIndex:
			help: nil.
		builder newButtonFor: self
			action: #refresh
			label: 'Refresh'
			help: nil.
	}.! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 17:19'!
dropList
	"Provides list contents for the drop list."
	| newsSources |
	
	newsSources := OrderedCollection new.
	
	dropList do: [ :i |
		newsSources add: (i at: 2)].

	^newsSources! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 19:42'!
getIndex
	"Returns currently selected list item in the headline list component. If nothing is selected, the index is 0."
	^index! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 17:21'!
getListIndex
	"Returns the currently selected item in the drop list. If nothing is selected, the index is 0."
	^listIndex! !

!RSSNewsReader methodsFor: 'initialization' stamp: 'MJK 7/5/2013 18:43'!
initialize
	"The constructor. Initializes instance variables. Fills the 'dropList' array with RSS URLs and names for the news sources."
	list := OrderedCollection new.
	index := 0.

	listIndex := 1.
	dropList := {
		{'http://yle.fi/uutiset/rss/uutiset.rss'. 'YLE Uutiset'}.
		{'http://store.steampowered.com/feeds/news.xml'. 'Steam News'}.
		{'http://feeds.bbci.co.uk/news/rss.xml'. 'BBC News'}.
	}.

	self buildGUI.
	self refresh.! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 18:44'!
list
	"Returns the headline list."
	
	| headlines |
	
	headlines := OrderedCollection new.
	
	list do: [ :i |
		headlines add: (i at: 2)].
	
	^headlines! !

!RSSNewsReader methodsFor: 'user-interface' stamp: 'MJK 7/5/2013 17:16'!
open
	"Opens the GUI contents in a windows. Defines dimensions for the window."
	(contents openInWindowLabeled: 'News Reader') extent: 400@500.! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 18:43'!
refresh
	| url source xml title addr |
	
	url := (dropList at: listIndex) at: 1.
	
	source := url asUrl retrieveContents.
	xml := XMLDOMParser parseDocumentFrom: source.
	index := 0.
	self changed: #getIndex.
	list removeAll.
	self changed: #list.
	xml allElementsNamed: #item do: [ :element |
		title := ''.
		addr := ''.
		element elementsDo: [ :element2 |
			element2 name = 'title' ifTrue: [
				title := element2 contentString.
			].
			element2 name = 'link' ifTrue: [
				addr := element2 contentString.
			].
		].
		list add: { addr. title }.
	].
	self changed: #list.! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 18:46'!
setIndex: anInteger
	"Sets the currently selected item in the headline list. 0 = nothing has been selected."
	index := anInteger.
	self changed: #getIndex.
	index = 0 ifFalse: [ Win32Shell new shellOpen: ((list at: index) at: 1) ].! !

!RSSNewsReader methodsFor: 'events' stamp: 'MJK 7/5/2013 18:46'!
setListIndex: anInteger
	"Sets the currently selected item in the dropdown list. 0 = nothing has been selected."
	listIndex := anInteger.
	listIndex = 0 ifFalse: [ self refresh ].! !
